@@ -19,6 +19,7 @@
 
 #include <config.h>
 
+#include <math.h>
 #include <string.h>
 
 #include "gxps-brush.h"
@@ -58,6 +59,13 @@ gxps_brush_new (GXPSRenderContext *ctx)
         return brush;
 }
 
+static gdouble
+gxps_transform_hypot (const cairo_matrix_t *matrix, double dx, double dy)
+{
+	cairo_matrix_transform_distance (matrix, &dx, &dy);
+	return hypot (dx, dy);
+}
+
 void
 gxps_brush_free (GXPSBrush *brush)
 {
@@ -859,6 +867,7 @@ brush_start_element (GMarkupParseContext  *context,
                 cairo_rectangle_t viewport, viewbox;
                 cairo_matrix_t matrix;
                 cairo_extend_t extend = CAIRO_EXTEND_NONE;
+                double width, height;
                 gint i;
 
                 cairo_matrix_init_identity (&matrix);
@@ -911,15 +920,21 @@ brush_start_element (GMarkupParseContext  *context,
 
                 /* TODO: check required values */
 
-                /* Clip to viewport so that push group
-                 * will create a surface with the viport size
-                 */
+                width = gxps_transform_hypot (&matrix, viewport.width, 0);
+                height = gxps_transform_hypot (&matrix, 0, viewport.height);
+
                 cairo_save (brush->ctx->cr);
-                cairo_rectangle (brush->ctx->cr, viewbox.x, viewbox.y, viewbox.width, viewbox.height);
+                cairo_rectangle (brush->ctx->cr, 0, 0, width, height);
                 cairo_clip (brush->ctx->cr);
                 cairo_push_group (brush->ctx->cr);
+                cairo_translate (brush->ctx->cr, -viewbox.x, -viewbox.y);
+                cairo_scale (brush->ctx->cr, width / viewbox.width, height / viewbox.height);
                 visual = gxps_brush_visual_new (brush, &viewport, &viewbox);
                 visual->extend = extend;
+                cairo_matrix_init (&visual->matrix, viewport.width / width, 0, 0,
+                                   viewport.height / height, viewport.x, viewport.y);
+                cairo_matrix_multiply (&visual->matrix, &visual->matrix, &matrix);
+                cairo_matrix_invert (&visual->matrix);
                 sub_ctx = g_slice_new0 (GXPSRenderContext);
                 sub_ctx->page = brush->ctx->page;
                 sub_ctx->cr = brush->ctx->cr;
@@ -947,60 +962,64 @@ brush_end_element (GMarkupParseContext  *context,
         } else if (strcmp (element_name, "RadialGradientBrush") == 0) {
                 g_markup_parse_context_pop (context);
         } else if (strcmp (element_name, "ImageBrush") == 0) {
-                GXPSBrushImage  *image;
-                cairo_surface_t *surface;
+                GXPSBrushImage  *brush_image;
+                GXPSImage       *image;
                 GError          *err = NULL;
 
-                image = g_markup_parse_context_pop (context);
+                brush_image = g_markup_parse_context_pop (context);
 
                 GXPS_DEBUG (g_message ("set_fill_pattern (image)"));
-                surface = gxps_page_get_image (brush->ctx->page, image->image_uri, &err);
-                if (surface) {
-                        cairo_matrix_t matrix, port_matrix;
-                        gdouble        x_scale, y_scale;
+                image = gxps_page_get_image (brush->ctx->page, brush_image->image_uri, &err);
+                if (image) {
+                        cairo_matrix_t   matrix;
+                        gdouble          x_scale, y_scale;
+                        cairo_surface_t *clip_surface;
 
-                        image->brush->pattern = cairo_pattern_create_for_surface (surface);
-                        cairo_pattern_set_extend (image->brush->pattern, image->extend);
-                        cairo_surface_destroy (surface);
+                        /* viewbox units is 1/96 inch, convert to pixels */
+                        brush_image->viewbox.x *= image->res_x / 96;
+                        brush_image->viewbox.y *= image->res_y / 96;
+                        brush_image->viewbox.width *= image->res_x / 96;
+                        brush_image->viewbox.height *= image->res_y / 96;
 
-                        cairo_matrix_init (&port_matrix,
-                                           image->viewport.width,
-                                           0, 0,
-                                           image->viewport.height,
-                                           image->viewport.x,
-                                           image->viewport.y);
-                        cairo_matrix_multiply (&port_matrix, &port_matrix, &image->matrix);
+                        clip_surface = cairo_surface_create_for_rectangle (image->surface,
+                                                                           brush_image->viewbox.x,
+                                                                           brush_image->viewbox.y,
+                                                                           brush_image->viewbox.width,
+                                                                           brush_image->viewbox.height);
+                        brush_image->brush->pattern = cairo_pattern_create_for_surface (clip_surface);
+                        cairo_pattern_set_extend (brush_image->brush->pattern, brush_image->extend);
 
-                        x_scale = image->viewbox.width / port_matrix.xx;
-                        y_scale = image->viewbox.height / port_matrix.yy;
+                        x_scale = brush_image->viewport.width / brush_image->viewbox.width;
+                        y_scale = brush_image->viewport.height / brush_image->viewbox.height;
                         cairo_matrix_init (&matrix, x_scale, 0, 0, y_scale,
-                                           -port_matrix.x0 * x_scale,
-                                           -port_matrix.y0 * y_scale);
-                        cairo_pattern_set_matrix (image->brush->pattern, &matrix);
+                                           brush_image->viewport.x, brush_image->viewport.y);
+                        cairo_matrix_multiply (&matrix, &matrix, &brush_image->matrix);
+                        cairo_matrix_invert (&matrix);
+                        cairo_pattern_set_matrix (brush_image->brush->pattern, &matrix);
 
                         if (brush->opacity != 1.0) {
                                 cairo_push_group (brush->ctx->cr);
-                                cairo_set_source (brush->ctx->cr, image->brush->pattern);
-                                cairo_pattern_destroy (image->brush->pattern);
+                                cairo_set_source (brush->ctx->cr, brush_image->brush->pattern);
+                                cairo_pattern_destroy (brush_image->brush->pattern);
                                 cairo_paint_with_alpha (brush->ctx->cr, brush->opacity);
-                                image->brush->pattern = cairo_pop_group (brush->ctx->cr);
+                                brush_image->brush->pattern = cairo_pop_group (brush->ctx->cr);
                         }
 
-                        if (cairo_pattern_status (image->brush->pattern)) {
-                                GXPS_DEBUG (g_debug ("%s", cairo_status_to_string (cairo_pattern_status (image->brush->pattern))));
-                                cairo_pattern_destroy (image->brush->pattern);
-                                image->brush->pattern = NULL;
+                        if (cairo_pattern_status (brush_image->brush->pattern)) {
+                                GXPS_DEBUG (g_debug ("%s", cairo_status_to_string (cairo_pattern_status (brush_image->brush->pattern))));
+                                cairo_pattern_destroy (brush_image->brush->pattern);
+                                brush_image->brush->pattern = NULL;
                         }
+                        cairo_surface_destroy (clip_surface);
                 } else if (err) {
                         GXPS_DEBUG (g_debug ("%s", err->message));
                         g_error_free (err);
                 }
-                gxps_brush_image_free (image);
+                gxps_brush_image_free (brush_image);
         } else if (strcmp (element_name, "VisualBrush") == 0) {
                 GXPSRenderContext *sub_ctx;
                 GXPSBrushVisual   *visual;
-                cairo_matrix_t     matrix, port_matrix;
-                gdouble            x_scale, y_scale;
+                cairo_matrix_t     matrix;
 
                 sub_ctx = g_markup_parse_context_pop (context);
                 visual = sub_ctx->visual;
@@ -1011,20 +1030,8 @@ brush_end_element (GMarkupParseContext  *context,
                 /* Undo the clip */
                 cairo_restore (brush->ctx->cr);
                 cairo_pattern_set_extend (visual->brush->pattern, visual->extend);
-                cairo_matrix_init (&port_matrix,
-                                   visual->viewport.width,
-                                   0, 0,
-                                   visual->viewport.height,
-                                   visual->viewport.x,
-                                   visual->viewport.y);
-                cairo_matrix_multiply (&port_matrix, &port_matrix, &visual->matrix);
-
-                x_scale = visual->viewbox.width / port_matrix.xx;
-                y_scale = visual->viewbox.height / port_matrix.yy;
-                cairo_matrix_init (&matrix, x_scale, 0, 0, y_scale,
-                                   -port_matrix.x0 * x_scale,
-                                   -port_matrix.y0 * y_scale);
-
+                cairo_pattern_get_matrix (visual->brush->pattern, &matrix);
+                cairo_matrix_multiply (&matrix, &visual->matrix, &matrix);
                 cairo_pattern_set_matrix (visual->brush->pattern, &matrix);
                 if (cairo_pattern_status (visual->brush->pattern)) {
                         GXPS_DEBUG (g_debug ("%s", cairo_status_to_string (cairo_pattern_status (visual->brush->pattern))));
