@@ -20,12 +20,15 @@
 #include <config.h>
 
 #include <glib.h>
+#include <locale.h>
 #include G_STRINGIFY(CONVERTER_HEADER)
 
 gint main (gint argc, gchar **argv)
 {
         GXPSConverter *converter;
 
+        setlocale (LC_ALL, "");
+
         g_type_init ();
 
         converter = GXPS_CONVERTER (g_object_new (CONVERTER_TYPE, NULL));
